#include "stdafx.h"
#include "pteTable.h"

pteTable::pteTable()
{
	CellWidth = 45;
	AtomicNumberXOffset = 2;
	AtomicNumberYOffset = 2;
	AtomicSymbolXOffset = 10;
	AtomicSymbolYOffset = 16;
	FontFace = "Arial";
	FontSize = 110;
};


pteTable::~pteTable()
{
};


/* ************************************** */
/* Initialize the groups                  */
/* ************************************** */
void pteTable::InitGroups()
{
	// Load the group settings from the .INI file
	// For now, hardcode them
	GroupBGColors[0] = RGB(255, 255, 196);
	GroupBGColors[1] = RGB(255, 196, 255);
	GroupBGColors[2] = RGB(196, 255, 255);
	GroupBGColors[3] = RGB(255, 196, 196);
	GroupBGColors[4] = RGB(196, 255, 196);
	GroupBGColors[5] = RGB(196, 196, 255);
	GroupBGColors[6] = RGB(255, 128, 255);
	GroupBGColors[7] = RGB(128, 255, 255);
	GroupBGColors[8] = RGB(196, 196, 196);
	GroupBGColors[9] = RGB(255, 255, 255);
}


/* ************************************** */
/* Query the DB and populate the elements */
/* ************************************** */
void pteTable::InitElements()
{
	CDatabase  db;
	db.OpenEx("DSN=PeriodicTableApp");
	CRecordset rs(&db);
	CString    data;
	int        position=0;

	rs.Open(AFX_DB_USE_DEFAULT_TYPE, (LPCSTR)"SELECT * FROM Elements");
	if(!rs.IsEOF())
	{
		while(!rs.IsEOF())
		{
			if(position > 117)
			{
				MessageBox(NULL, "DB Error", "There are too many records in the database!", MB_ICONSTOP);
				break;
			}
			rs.GetFieldValue("AtomicNumber", data);	
			Elements[position].SetAtomicNumber(data);
			rs.GetFieldValue("AtomicWeight", data);	
			Elements[position].SetAtomicWeight(data);
			rs.GetFieldValue("GroupNumber", data);	
			Elements[position].SetGroupNumber(data);
			rs.GetFieldValue("GroupName", data);	
			Elements[position].SetGroupName(data);
			rs.GetFieldValue("Period", data);	
			Elements[position].SetPeriod(data);
			rs.GetFieldValue("Block", data);	
			Elements[position].SetBlock(data);
			rs.GetFieldValue("CASRegistryID", data);	
			Elements[position].SetCASRegistryID(data);
			rs.GetFieldValue("ElementName", data);	
			Elements[position].SetElementName(data);
			rs.GetFieldValue("ElementSymbol", data);	
			Elements[position].SetElementSymbol(data);
			rs.GetFieldValue("DiscoveryDate", data);	
			Elements[position].SetDiscoveryDate(data);
			rs.GetFieldValue("DiscovererName", data);	
			Elements[position].SetDiscovererName(data);

			rs.GetFieldValue("Row", data);	
			Elements[position].SetRow(data);
			rs.GetFieldValue("Column", data);	
			Elements[position].SetColumn(data);

			rs.MoveNext();	
			position++;
		}
	}
	rs.Close();
	db.Close();

	BuildElementRects();
};


/* ******************************** */
/* Build the element's rectangless  */
/* ******************************** */
void pteTable::BuildElementRects()
{
	for(int x = 0; x < NUM_ELEMENTS; x++)
		Elements[x].BuildRect(CellWidth);
}


/* ******************************** */
/* Set the current element          */
/* ******************************** */
void pteTable::SetCurrentElement(int n)
{
	CurrentElement = n;
}


/* ******************************** */
/* Element array GET method         */
/* ******************************** */
pteElement pteTable::GetElement(int n)
{
	return Elements[n];
}


/* ******************************** */
/* ElementName GET method          */
/* ******************************** */
CString pteTable::GetElementName(int n)
{
	return Elements[n].GetElementName();
}


/* ******************************** */
/* Element find method              */
/* ******************************** */
int pteTable::FindElement(CPoint point)
{
	BOOL  MatchFound = FALSE;

	for(int x = 0; x < 118; x++)
	{
		// if you have found a match, store the match and exit loop
		if(Elements[x].CheckHit(point))
		{
			MatchFound = TRUE;
			break;
		}
	}

	if(MatchFound)
		return x;
	else
		return -1;
}


/* ******************************** */
/* Group BG Color access methods    */
/* ******************************** */
void pteTable::SetGroupBGColor(int n, COLORREF color)
{
	GroupBGColors[n] = color;
}

COLORREF pteTable::GetGroupBGColor(int n)
{
	return(GroupBGColors[n]);
}


/* ******************************** */
/* Element row/column methods       */
/* ******************************** */
int pteTable::GetRow(int element)
{
	return Elements[element].GetRow();
}
int pteTable::GetColumn(int element)
{
	return Elements[element].GetColumn();
}


/* ******************************** */
/* Element Number, Symbol & Group   */
/* ******************************** */
CString pteTable::GetAtomicSymbol(int element)
{
	return Elements[element].GetElementSymbol();
}
CString pteTable::GetAtomicNumber(int element)
{
	return Elements[element].GetAtomicNumber();
}
CString pteTable::GetGroup(int element)
{
	return Elements[element].GetGroupNumber();
}


/* ******************************** */
/* Font size methods                */
/* ******************************** */
void pteTable::SetFontSize(int size)
{
	FontSize = size;
}
int pteTable::GetFontSize()
{
	return FontSize;
}


/* ******************************** */
/* Font face methods                */
/* ******************************** */
void pteTable::SetFontFace(CString face)
{
	FontFace = face;
}
CString pteTable::GetFontFace()
{
	return FontFace;
}


/* ******************************** */
/* Cell Width methods               */
/* ******************************** */
int pteTable::GetCellWidth()
{
	return CellWidth;
}
void pteTable::SetCellWidth(int w)
{
	CellWidth = w;
}


/* ******************************** */
/* XY offsets for drawing text      */
/* ******************************** */
int pteTable::GetAtomicSymbolXOffset()
{
	return AtomicSymbolXOffset;
}
int pteTable::GetAtomicSymbolYOffset()
{
	return AtomicSymbolYOffset;
}
int pteTable::GetAtomicNumberXOffset()
{
	return AtomicNumberXOffset;
}
int pteTable::GetAtomicNumberYOffset()
{
	return AtomicNumberYOffset;
}


/* ********************************* */
/* Build the Invalid update region   */
/* ********************************* */
void pteTable::BuildUpdateRegion(CDialog* dlg, int group, int oldgroup) 
{
	for(int x = 0; x < NUM_ELEMENTS; x++)
	{
		if((atoi(Elements[x].GetGroupNumber()) == group) || (atoi(Elements[x].GetGroupNumber()) == oldgroup))
		{
			dlg->InvalidateRect(Elements[x].GetRect(), FALSE);	
		}
	}
}


/* ********************************* */
/* Build the Invalid update region   */
/* ********************************* */
void pteTable::BuildUpdateRegion(CDialog* dlg, int group) 
{
	for(int x = 0; x < NUM_ELEMENTS; x++)
	{
		if(atoi(Elements[x].GetGroupNumber()) == group)
		{
			dlg->InvalidateRect(Elements[x].GetRect(), FALSE);	
		}
	}
}


/* ******************************** */
/* Draw the table                   */
/* ******************************** */
void pteTable::Draw(CPaintDC* dc, int group)
{
	// declare and initialize the fonts
	CFont*  OldFont;
	CFont   NumberFont;
	CFont   SymbolFont;
	NumberFont.CreatePointFont(FontSize - 40, FontFace, NULL);
	SymbolFont.CreatePointFont(FontSize, FontFace, NULL);

	// declare and initialize the brushes
	CBrush* OldBrush;
	CBrush  GroupBrushes[NUM_GROUPS];
	for(int x = 0; x < NUM_GROUPS; x++)
		GroupBrushes[x].CreateSolidBrush(GetGroupBGColor(x));

	// declare and initialize the pens
	CPen*   OldPen;
	CPen    RedOutLinePen(PS_SOLID, 1, RGB(255, 0, 0));
	CPen    BlackOutLinePen(PS_SOLID, 1, RGB(0, 0, 0));

	// draw the elements
	for(x = 0; x < NUM_ELEMENTS; x++)
	{
		OldBrush = dc->SelectObject(&GroupBrushes[atoi(Elements[x].GetGroupNumber())]);

		// if you are drawing the active group, outline it in red, otherwise black
		if(atoi(Elements[x].GetGroupNumber()) == group)
			OldPen = dc->SelectObject(&RedOutLinePen);
		else
			OldPen = dc->SelectObject(&BlackOutLinePen);

		dc->Rectangle(((Elements[x].GetColumn() - 1) * CellWidth), ((Elements[x].GetRow() - 1) * CellWidth), ((Elements[x].GetColumn() - 1) * CellWidth) + CellWidth, ((Elements[x].GetRow() - 1) * CellWidth) + CellWidth);
		dc->SelectObject(OldBrush);
		dc->SelectObject(OldPen);
		dc->SetBkMode(TRANSPARENT);
		
		OldFont = dc->SelectObject(&NumberFont);
		dc->TextOut(((Elements[x].GetColumn() - 1) * CellWidth) + AtomicNumberXOffset, ((Elements[x].GetRow()    - 1) * CellWidth) + AtomicNumberYOffset, Elements[x].GetAtomicNumber(), Elements[x].GetAtomicNumber().GetLength());
		dc->SelectObject(&SymbolFont);
		dc->TextOut(((Elements[x].GetColumn() - 1) * CellWidth) + AtomicSymbolXOffset, ((Elements[x].GetRow()    - 1) * CellWidth) + AtomicSymbolYOffset, Elements[x].GetElementSymbol(), Elements[x].GetElementSymbol().GetLength());
		dc->SelectObject(OldFont);
	}

	// clean-up fonts
	SymbolFont.DeleteObject();
	NumberFont.DeleteObject();

	// clean-up brushes
	for(x = 0; x < NUM_GROUPS; x++)
		GroupBrushes[x].DeleteObject();

	// clean-up pens
	RedOutLinePen.DeleteObject();
	BlackOutLinePen.DeleteObject();
}
